//********************************************************************
//*** ResourcePool/SQL/QLException.t.cc
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: SQLException.t.cc,v 1.10 2005/01/06 16:27:56 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include "SQLException.hh"
#include "ResourcePool/NoFailoverException.hh"
#include <iostream>

std::auto_ptr<RP> _rp;

void test_init()
{
	if (_rp.get() == 0) {
		_rp.reset(new RP(getFactory()));
	}
}


void test_SQLException_init()
{
	test_init();
	init(*_rp);
}

void test_SQLException_simple_constraints()
{
	RP::Command::SQLExecute ins("insert into hirsch (id, ukn) values (?, ?)");
	std::string nul("notnull");
	ins.bind(0, 1001);
	ins.bind(1, nul, true);
	_rp->execute(ins);
	// UNIQE (PK) constraint
	try {
		_rp->execute(ins);
		assertNotReach();
	} catch (fatalmind::SQL::SQLException e) {
		try {
			assertEqual(e.getSQLState(), "23000");
			assertTrue(e.isSQLState("23000"));
			assertTrue(e.isSQLStateClass("23"));
			assertFalse(e.isFailover());
		} catch (...) {
			std::cerr << e.message() << std::endl;
			throw;
		}
	}
	// NOT NULL CONSTRAINT
	try {
		ins.bind(0, 1002, true);
		_rp->execute(ins);
		assertNotReach();
	} catch (fatalmind::SQL::SQLException e) {
		try {
			assertEqual(e.getSQLState(), "23000");
			assertTrue(e.isSQLState("23000"));
			assertTrue(e.isSQLStateClass("23"));
			assertFalse(e.isFailover());
		} catch (...) {
			std::cerr << e.message() << std::endl;
			throw;
		}
	}
}

void priv_test_SQLException_syntax(RP& p, const std::string& sql) {
	RP::Command::SQLExecute ins(sql);
	try {
		p.execute(ins);
		assertNotReach();
	} catch (fatalmind::SQL::SQLException e) {
		try {
			assertEqual(e.getSQLState(), "07000");
			assertTrue(e.isSQLState("07000"));
			assertTrue(e.isSQLStateClass("07"));
			assertFalse(e.isFailover());
		} catch (...) {
			std::cerr << e.message() << std::endl;
			throw;
		}
	}
}
void test_SQLException_syntax() {
	priv_test_SQLException_syntax(*_rp, "inser into hirsch (id, ukn) values (1, 'xx')");
	priv_test_SQLException_syntax(*_rp, "insert into hirsch (ukn) values (1, 'xx')");
	priv_test_SQLException_syntax(*_rp, "insert into hirsch (id, ukn) values (1, 'xx)");
	priv_test_SQLException_syntax(*_rp, "insert into irsch (id, ukn) values (1, 'xx)");
	priv_test_SQLException_syntax(*_rp, "insert into irsch (id, uk) values (1, 'xx)");
}

void priv_test_SQLException_SQLSelectRow(RP& p, const std::string& sql, const char* const val) {
	RP::Command::SQLSelectRow srow(sql);
	int i;
	srow.bindout(0, i);
	try {
		p.execute(srow);
		assertNotReach();
	} catch (fatalmind::SQL::SQLException e) {
		try {
			assertEqual(e.getSQLState(), val);
			assertFalse(e.isFailover());
		} catch (...) {
			std::cerr << e.message() << std::endl;
			throw;
		}
	}
}

void test_SQLException_SQLSelectRow() {
	RP::Command::SQLExecute ins("insert into hirsch (id, ukn) values (15, '')");
	_rp->execute(ins);
	priv_test_SQLException_SQLSelectRow(*_rp, "select id from hirsch where id = 999999", "02000");
	priv_test_SQLException_SQLSelectRow(*_rp, "select id from hirsch", "22000");
}

void test_SQLException_SQLSelectRow_wrong_bindout() {
	RP::Command::SQLSelectRow srow("select * from hirsch where id = 1001");
	try {
		_rp->execute(srow);
		assertNotReach();
	} catch (fatalmind::SQL::SQLException e) {
		try {
			assertEqual(e.getSQLState(), "07001");
			assertFalse(e.isFailover());
		} catch (...) {
			std::cerr << e.message() << std::endl;
			throw;
		}
	}
	std::string str;
	srow.bindout(1, str);
	try {
		_rp->execute(srow);
		assertNotReach();
	} catch (fatalmind::SQL::SQLException e) {
		try {
			assertEqual(e.getSQLState(), "07001");
			assertFalse(e.isFailover());
		} catch (...) {
			std::cerr << e.message() << std::endl;
			throw;
		}
	}
}

void test_SQLException_SQLSelectRow_wrong_bind() {
	RP::Command::SQLExecute txt("update hirsch set id = ? where id = ?");
	try {
		_rp->execute(txt);
		assertNotReach();
	} catch (fatalmind::SQL::SQLException e) {
		try {
			assertEqual(e.getSQLState(), "07001");
			assertFalse(e.isFailover());
		} catch (...) {
			std::cerr << e.message() << std::endl;
			throw;
		}
	}
	txt.bind(1, 2);
	try {
		_rp->execute(txt);
		assertNotReach();
	} catch (fatalmind::SQL::SQLException e) {
		try {
			assertEqual(e.getSQLState(), "07001");
			assertFalse(e.isFailover());
		} catch (...) {
			std::cerr << e.message() << std::endl;
			throw;
		}
	}
}

void test_SQLException_cleanup() {
	drop(*_rp);
}
